#include <GL/glut.h>
#include <stdlib.h>		
#include <stdio.h>
#include <string.h>

static char label[100];
int w,h;
int frame=0, time=0, timebase=0;
float fps;
float angulo = 0.0;		


void inline drawString (char *s)
{
 unsigned int i;
 for (i = 0; i < strlen (s); i++)
    glutBitmapCharacter (GLUT_BITMAP_HELVETICA_10, s[i]);
}


void setOrthographicProjection() 
{
 glMatrixMode(GL_PROJECTION);
 glPushMatrix();
 glLoadIdentity();
 gluOrtho2D(0, w, 0, h);
 glScalef(1, -1, 1);
 glTranslatef(0, -h, 0);
 glMatrixMode(GL_MODELVIEW);
}


void resetPerspectiveProjection() 
{
 glMatrixMode(GL_PROJECTION);
 glPopMatrix();
 glMatrixMode(GL_MODELVIEW);
}


void init(void)
{
 GLfloat light_ambient[] = { 0.75, 0.75, 0.75, 1.0 };
 GLfloat light_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
 GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
 GLfloat light_position[] = { 0.1, 0.25, 1.0, 0.0 };

 glLightfv (GL_LIGHT0, GL_AMBIENT, light_ambient);
 glLightfv (GL_LIGHT0, GL_DIFFUSE, light_diffuse);
 glLightfv (GL_LIGHT0, GL_SPECULAR, light_specular);
 glLightfv (GL_LIGHT0, GL_POSITION, light_position);
    
 glEnable (GL_LIGHTING);
 glEnable (GL_LIGHT0);
 glShadeModel(GL_SMOOTH);
 glEnable(GL_DEPTH_TEST);			    
 glDepthFunc(GL_LEQUAL);	
 glClearColor(0.0f, 0.0f, 0.0f, 1.0f);		    
}


void display(void)
{
 GLfloat mat_ambient[] = {0.1, 0.1, 0.1, 1.0f}; 
 GLfloat mat_diffuse[] = {0.308, 0.615, 0.824, 1.0f};
 GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0f};
 
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);   

 glPushMatrix(); 
 glLoadIdentity();
 glTranslatef(0.0f,0.0f,-10.0f);	
 glRotatef(angulo, 0.0f, 1.0f, 0.0f);
 glMaterialfv (GL_FRONT, GL_AMBIENT, mat_ambient);
 glMaterialfv (GL_FRONT, GL_DIFFUSE, mat_diffuse);
 glMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular);
 glMaterialf (GL_FRONT, GL_SHININESS, 50.0f);
 glutSolidTeapot(2.5);
 glPopMatrix();

 setOrthographicProjection();
	
 glPushMatrix();
 glPushAttrib( GL_LIGHTING_BIT );
 glDisable( GL_LIGHTING );
 glLoadIdentity();
 glColor3f(1.0f,1.0f,1.0f);	
 sprintf(label,"FPS:%5.2f",fps);
 glRasterPos2f(30.0,30.0);
 drawString (label);
 glPopAttrib();
 glPopMatrix();

 resetPerspectiveProjection();
 
 glutSwapBuffers();
}


void reshape (int width, int height)
{
 if (height==0)					
	height=1;
 w=width;
 h=height;
 glViewport(0,0,w,h);						
 glMatrixMode(GL_PROJECTION);						
 glLoadIdentity();									
 gluPerspective(60.0f,(GLfloat)w/(GLfloat)h,0.1f,100.0f);
 glMatrixMode(GL_MODELVIEW);							
 glLoadIdentity();	 
}


void keyboard(unsigned char key, int x, int y)
{
 switch (key) 
   {
   case 27: exit(0);
             break;
   }
}                                                                


void Idle(void)
{
 angulo += 0.5;
 frame++;
 time=glutGet(GLUT_ELAPSED_TIME);
	if (time - timebase > 1000) 
		{
		fps=frame*1000.0/(time-timebase);
		timebase = time;		
		frame = 0;
		}
 display();
}

int main(int argc, char** argv)
{
 glutInit(&argc, argv);
 glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
 glutInitWindowSize (300, 300); 
 glutInitWindowPosition (0, 0);
 glutCreateWindow ("Rotacion 3D (FPS)");
 init ();
 glutDisplayFunc(display); 
 glutReshapeFunc(reshape);
 glutKeyboardFunc(keyboard);
 glutIdleFunc(Idle);
 glutMainLoop();
 return 0;
}
